﻿# -*- coding: UTF-8 -*-

# App module for Perky Duck
# http://www.duxburysystems.com

import addonHandler
import appModuleHandler
import config
import os
import api
import ui
import controlTypes
import speech
import textInfos
import eventHandler
from NVDAObjects.behaviors import EditableText
from itertools import combinations
from collections import OrderedDict
from configobj import ConfigObj

addonHandler.initTranslation()

# Two functions to build settings file

def getBrailleDotsList():
	brailleDots = xrange(1, 7)
	brailleChars = []
	orderedChars = []
	for x in brailleDots:
		brailleChars.extend(combinations(brailleDots, x))
	for char in brailleChars:
		orderedChar = map(str, list(char))
		dot = "".join(orderedChar)
		orderedChars.append(dot)
	return orderedChars

def deleteReplacedChars():
	charsDic = OrderedDict()
	for char in getBrailleDotsList():
		charsDic[char] = ""
	return charsDic

iniFileName = os.path.join(os.path.dirname(__file__), "..", "settings", "braille.ini")
conf = ConfigObj(iniFileName, encoding="UTF-8", indent_type="\t", list_values=False)
#conf["symbols"] = deleteReplacedChars()
#conf.write()

def getChar(dots):
	global conf, iniFileName
	conf = ConfigObj(iniFileName)
	char = conf["symbols"][dots]
	return char

def getDots(char):
	global conf, iniFileName
	conf = ConfigObj(iniFileName)
	dots = ""
	charsDic = conf["symbols"]
	for k, v in charsDic.iteritems():
		if v == char:
			dots = k
			break
	return dots

class AppModule(appModuleHandler.AppModule):

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		if obj.windowClassName == "DBT Window":
			clsList.insert(0, BrailleDocument)

	def event_caret(self, obj, nextHandler):
		nextHandler()
		if isinstance(obj, BrailleDocument) and obj.writeMode == 1:
			if obj.spaceKeyPressed == True:
				obj.spaceKeyPressed = False
				return
			obj.reportPreviousCharacter()

class BrailleDocument(EditableText):

	writeMode = 0
	spaceKeyPressed = False
	oldFollowCaret = config.conf["reviewCursor"]["followCaret"]
	oldFollowFocus = config.conf["reviewCursor"]["followFocus"]

	def reportPreviousCharacter(self):
		charInfo = api.getReviewPosition().copy()
		oldCharInfo = charInfo.copy()
		res=charInfo.move(textInfos.UNIT_CHARACTER,-1)
		api.setReviewPosition(charInfo.copy())
		charInfo.expand(textInfos.UNIT_CHARACTER)
		char = charInfo.text
		replacedChar = getDots(char)
		api.setReviewPosition(oldCharInfo.copy())
		if replacedChar == "":
			ui.message(_("Character:"))
			speech.speakTextInfo(charInfo,unit=textInfos.UNIT_CHARACTER,reason=controlTypes.REASON_CARET)
			return
		ui.message(replacedChar)

	def _caretMovementScriptHelper(self, gesture, unit):
		if self.writeMode == 1:
			config.conf["reviewCursor"]["followCaret"] = self.oldFollowCaret
			config.conf["reviewCursor"]["followFocus"] = self.oldFollowFocus
			self.writeMode = 0
			ui.message(_("Write mode {writeModeState}").format(writeModeState=_("off")))
		try:
			info=self.makeTextInfo(textInfos.POSITION_CARET)
		except:
			gesture.send()
			return
		bookmark=info.bookmark
		gesture.send()
		caretMoved,newInfo=self._hasCaretMoved(bookmark) 
		if not caretMoved and self.shouldFireCaretMovementFailedEvents:
			eventHandler.executeEvent("caretMovementFailed", self, gesture=gesture)
		self._caretScriptPostMovedHelper(unit,newInfo)

	def script_space(self, gesture):
		if self.writeMode == 1:
			self.spaceKeyPressed = True
			ui.message(gesture.displayName)
		gesture.send()

	def script_toggleWriteMode(self, gesture):
		if self.writeMode == 0:
			self.oldFollowCaret = config.conf["reviewCursor"]["followCaret"]
			self.oldFollowFocus = config.conf["reviewCursor"]["followFocus"]
			config.conf["reviewCursor"]["followCaret"] = True
			config.conf["reviewCursor"]["followFocus"] = True
			self.writeMode = 1
			self.spacePressed = False
			ui.message(_("Write mode {writeModeState}").format(writeModeState=_("on")))
		else:
			config.conf["reviewCursor"]["followCaret"] = self.oldFollowCaret
			config.conf["reviewCursor"]["followFocus"] = self.oldFollowFocus
			self.writeMode = 0
			ui.message(_("Write mode {writeModeState}").format(writeModeState=_("off")))
	script_toggleWriteMode.__doc__ = _("Turns on and off the write mode.")

	def script_sayPreviousCharacter(self, gesture):
		if self.writeMode == 0:
			gesture.send()
			return
		self.reportPreviousCharacter()
	script_sayPreviousCharacter.__doc__ = _("Reports the dots attached to the previous character.")

	def script_reportCharDots(self, gestures):
		info = api.getReviewPosition().copy()
		info.expand(textInfos.UNIT_CHARACTER)
		char = info.text
		ui.message(getDots(char))
	script_reportCharDots.__doc__ = _("Announces the dots associated with the character under the review cursor.")

	__gestures = {
		"kb:control+shift+NVDA+b": "toggleWriteMode",
		"kb:space": "space",
		"kb:NVDA+control+shift+n": "sayPreviousCharacter",
		"kb:NVDA+control+shift+m": "reportCharDots",
	}