﻿# -*- coding: UTF-8 -*-

# Gui for Perky add-on

import addonHandler
import globalPluginHandler
import sys
import os
import languageHandler
appModuleDir = os.path.join(os.path.dirname(__file__), "..", "appModules")
sys.path.append(appModuleDir)
import perky
del sys.path[-1]
import wx
import gui
from configobj import ConfigObj
from logHandler import log

addonHandler.initTranslation()

class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(globalPluginHandler.GlobalPlugin, self).__init__()
		self.prefs = gui.mainFrame.sysTrayIcon.preferencesMenu
		self.help = gui.mainFrame.sysTrayIcon.helpMenu
		self.settingsItem = self.prefs.Append(wx.ID_ANY, _("Settings for Per&ky..."), _("Set the configuration for Perky app module"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onSettings, self.settingsItem)
		self.helpItem = self.help.Append(wx.ID_ANY, _("Help for Per&ky"), _("Open documentation for Perky app module"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onAbout, self.helpItem)

	def terminate(self):
		try:
			self.prefs.RemoveItem(self.settingsItem)
			self.help.RemoveItem(self.helpItem)
		except wx.PyDeadObjectError:
			pass

	def onSettings(self, evt):
		d = wx.TextEntryDialog(gui.mainFrame, _("Enter a dots pattern"), _("Braille pattern"), defaultValue="123456")
		gui.mainFrame.prePopup()
		try:
			result = d.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			pattern = d.GetValue()
			if pattern:
				if pattern not in perky.getBrailleDotsList():
					wx.CallAfter(gui.messageBox, pattern, _("Invalid pattern!"), wx.OK|wx.ICON_ERROR)
				else:
					d = wx.TextEntryDialog(gui.mainFrame, _("Enter a character"), _("Character to associate with %s" % pattern), defaultValue="") # perky.getChar(pattern))
					gui.mainFrame.prePopup()
					try:
						result = d.ShowModal()
					except AttributeError:
						pass
					gui.mainFrame.postPopup()
					if result == wx.ID_OK:
						character = d.GetValue()
						if len(character) >1:
							wx.CallAfter(gui.messageBox, character, _("Invalid character!"), wx.OK|wx.ICON_ERROR)
						else:
							#perky.conf = ConfigObj(perky.iniFileName, encoding="UTF-8", indent_type="\t", list_values=False)
							perky.conf["symbols"][pattern] = character
							try:
								perky.conf.write()
								log.info("Perky app module configuration saved")
							except Exception, e:
								log.warning("Could not save Perky app module configuration")
								log.debugWarning("", exc_info=True)
								raise e
				if gui.messageBox(_("Do you want to associate another pattern with a character?"),
					_("Continue or finish"),
					wx.YES|wx.NO|wx.ICON_QUESTION)==wx.YES:
						self.onSettingsCallback()

	def onSettingsCallback(self):
		self.onSettings(None)

	def getDocFolder(self):
		langs = [languageHandler.getLanguage(), "en"]
		for lang in langs:
			docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", lang)
			if os.path.isdir(docFolder):
				return docFolder
			if "_" in lang:
				tryLang = lang.split("_")[0]
				docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", tryLang)
				if os.path.isdir(docFolder):
					return docFolder
				if tryLang == "en":
					break
			if lang == "en":
				break
		return None

	def getDocPath(self, docFileName):
		docPath = self.getDocFolder()
		if docPath is not None:
			docFile = os.path.join(docPath, docFileName)
			if os.path.isfile(docFile):
				docPath = docFile
		return docPath

	def onAbout(self, evt):
		try:
			os.startfile(self.getDocPath("readme.txt"))
		except WindowsError:
			pass

